%Oscillogramma di Cortocircuito della
%Macchina Sincrona

load I1cc.mat;      %Carica i valori nel vettore I1cc
I1=100.*I1cc';      %Trasforma i valori in Ampere
N=length(I1);       %Numero di Punti campionati
Tmax=(N-1)*0.1;     %Tempo massimo di Camp. [ms]
t=0:Tmax/N:Tmax-Tmax/N; %Variabile Temporale [ms]

E0=105; %Tensione di Fase a vuoto [V]

[Imax,Ind] = max(I1);   %Determina il picco massimo
Ic=Imax;
Idx=Ind;
NN=40;  %Numero di Picchi da considerare

%Il procedimento per ottenere la sequenza dei picchi di corrente
% il seguente: si ricava il primo picco mediante la funzione "max"
%la quale fornisce il valore del primo picco e la sua posizione
%all'interno del vettore I1.Si considera quindi il vettore I1 a
%partire dall'indice, corrispondente al picco appena trovato, pi
%100; ci corrisponde a spostarsi sull'asse dei tempi di circa 10 ms.
%Si esegue nuovamente l'istruzione "max" per il vettore restante
%ottenendo cos tutti i valori di picco fino all'NN-esimo!


for i=1:NN-1
     [Imax1,Ind1] = max(I1(Ind+100:N));
     Ic=[Ic Imax1];  %Vettore dei valori di Picco POSITIVI
     Ind=Ind+99+Ind1;   
     Idx=[Idx Ind];  %Vettore degli indici dei picchi POS.
end

%Si opera analogamente per i picchi negativi
%Invertendo il vettore I1, applicando lo stesso
%algoritmo ed infine reinvertendo nuovamente.

I1=-I1; %Inverte il vettore
[Imin,Ind] = max(I1);   %Determina il picco massimo
Im=Imin;
Idxm=Ind;

for i=1:NN-1
     [Imin1,Ind1] = max(I1(Ind+100:N));
     Im=[Im Imin1];  %Vettore dei valori di Picco     
     Ind=Ind+99+Ind1;   
     Idxm=[Idxm Ind];  %Vettore degli indici dei picchi NEG.
end

Im=-Im;     %Vettore dei valori di Picco NEGATIVI
I1=-I1;     %Riporta il vettore I1 ai valori effettivi


tc=Idx*(Tmax/N);    %Istanti dei Picchi POSITIVI [ms]
tm=Idxm*(Tmax/N);   %Istanti dei Picchi NEGATIVI [ms]

tc0=tc-tc(1);    %Considera come istante iniziale t0=0
tm0=tm-tc(1);    %l'istante in cui si verifica il primo picco +

%DETERMINAZIONE della FREQUENZA
for n=1:NN-1
    dtc=tc(n+1)-tc(n);
    Dtc(n)=dtc;
end

T=mean(Dtc);    %Periodo effettivo!
f=1000/T;       %Frequenza effettiva!


%DETERMINAZIONE DELLA CORRENTE DI REGIME
I1b=I1(N-3000:N);   %Considera gli ultimi 3000 punti 
                    %di Campionamento (c.a. 300 ms)
Ireg=max(I1b);
                    

tcm=sort([tc0 tm0]);    %Istanti dei picchi + e -
Imm=-Im;    %Picchi negativi cambiati di segno

Icm=[Ic(1) Imm(1)];
for nn=2:NN
    Icm=[Icm Ic(nn) Imm(nn)];
end
%Icm  il vettore dei picchi complessivi (+ e -)

%DETERMINAZIONE DEL TRANSITORIO
%Per avere una maggiore precisione nella
%determinazione dei parametri del Transitorio
%si sfruttano sia i picchi positivi che quelli
%negativi, cos da avere a disposizione pi informazioni.

p=16;   %Si parte dal 16 picco
tcmp=tcm(p:2*NN);
Icmp=Icm(p:2*NN)-Ireg;

lnIcmp=log(Icmp);   %Si esegue il logaritmo, per linearizzare
%cftool(tcmp,lnIcmp);
%Risultati del fitting
Kt=62.2;
T1=200;
yfit=log(Kt)-tc0/T1;


%Per ottenere la componente
%Subtrans.+Unidirez. basta
%sottrarre ai picchi positivi
%la componente Trans. e quella
%di regime:
y1=Ic-Kt*exp(-tc0./T1)-Ireg;
%cftool(tc0,y1);
%Risultati del fitting y1
Ks=52.3;
T2=9.2;
Ku=11.5;
Tu=53.7;


%Determinazione delle Reattanze
V=sqrt(2)*E0;
X=V/Ireg;
X1=V/(Kt+Ireg);
X2=V/(Ks+Kt+Ireg);